@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2019 Inria.  All rights reserved.
+ * Copyright © 2009-2020 Inria.  All rights reserved.
  * Copyright © 2009-2011 Université Bordeaux
  * Copyright © 2009-2010 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -66,10 +66,34 @@ static int intersectdepth = -1;
 static int hiernblevels = 0;
 static int *hierdepth = NULL;
 static int showobjs = 0;
-static int no_smt = 0;
+static int no_smt = -1;
 static int singlify = 0;
 static int taskset = 0;
 
+static int
+hwloc_calc_intersects_set(hwloc_bitmap_t set, int use_nodeset, hwloc_obj_t obj)
+{
+  if (use_nodeset)
+    return hwloc_bitmap_intersects(set, obj->nodeset);
+  else
+    return hwloc_bitmap_intersects(set, obj->cpuset);
+}
+
+/* generalization of hwloc_get_next_obj_covering_cpuset_by_depth() which may also use nodeset instead of cpuset */
+static hwloc_obj_t
+hwloc_calc_get_next_obj_covering_set_by_depth(hwloc_topology_t topology,
+					      hwloc_bitmap_t set, int use_nodeset,
+					      int depth,
+					      hwloc_obj_t prev)
+{
+  hwloc_obj_t next = hwloc_get_next_obj_by_depth(topology, depth, prev);
+  if (!next)
+    return NULL;
+  while (next && !hwloc_calc_intersects_set(set, use_nodeset, next))
+    next = next->next_cousin;
+  return next;
+}
+
 static void
 hwloc_calc_hierarch_output(hwloc_topology_t topology, const char *prefix, const char *sep, hwloc_obj_t root, hwloc_bitmap_t set, int level)
 {
@@ -103,21 +127,11 @@ next:
 static int
 hwloc_calc_output(hwloc_topology_t topology, const char *sep, hwloc_bitmap_t set)
 {
-  if (no_smt) {
+  if (no_smt != -1 && !nodeseto) {
     if (hwloc_get_type_depth(topology, HWLOC_OBJ_CORE) == HWLOC_TYPE_DEPTH_UNKNOWN) {
       fprintf(stderr, "Topology has no Core object, ignoring --no-smt\n");
     } else {
-      hwloc_obj_t core = NULL;
-      while ((core = hwloc_get_next_obj_covering_cpuset_by_type(topology, set, HWLOC_OBJ_CORE, core)) != NULL) {
-	int firstpu = hwloc_bitmap_first(core->cpuset);
-	int hadpu = hwloc_bitmap_isset(set, firstpu);
-	assert(firstpu >= 0);
-	/* remove the entire core */
-	hwloc_bitmap_andnot(set, set, core->cpuset);
-	/* put back its first PU if it was there */
-	if (hadpu)
-	  hwloc_bitmap_set(set, firstpu);
-      }
+      hwloc_bitmap_singlify_per_core(topology, set, no_smt);
     }
   }
 
@@ -127,6 +141,7 @@ hwloc_calc_output(hwloc_topology_t topology, const char *sep, hwloc_bitmap_t set
   if (showobjs) {
     hwloc_bitmap_t remaining = hwloc_bitmap_dup(set);
     int first = 1;
+    assert(!nodeseto); /* disabled for now, not very useful since the hierarchy of nodes isn't complex */
     if (!sep)
       sep = " ";
     while (!hwloc_bitmap_iszero(remaining)) {
@@ -152,14 +167,14 @@ hwloc_calc_output(hwloc_topology_t topology, const char *sep, hwloc_bitmap_t set
   } else if (numberofdepth != -1) {
     unsigned nb = 0;
     hwloc_obj_t obj = NULL;
-    while ((obj = hwloc_get_next_obj_covering_cpuset_by_depth(topology, set, numberofdepth, obj)) != NULL)
+    while ((obj = hwloc_calc_get_next_obj_covering_set_by_depth(topology, set, nodeseto, numberofdepth, obj)) != NULL)
       nb++;
     printf("%u\n", nb);
   } else if (intersectdepth != -1) {
     hwloc_obj_t proc, prev = NULL;
     if (!sep)
       sep = ",";
-    while ((proc = hwloc_get_next_obj_covering_cpuset_by_depth(topology, set, intersectdepth, prev)) != NULL) {
+    while ((proc = hwloc_calc_get_next_obj_covering_set_by_depth(topology, set, nodeseto, intersectdepth, prev)) != NULL) {
       if (prev)
 	printf("%s", sep);
       printf("%u", logicalo ? proc->logical_index : proc->os_index);
@@ -255,7 +270,7 @@ int main(int argc, char *argv[])
     hwloc_topology_set_all_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_ALL); \
     hwloc_topology_set_flags(topology, flags); \
     if (input) { \
-      err = hwloc_utils_enable_input_format(topology, input, &input_format, verbose, callname); \
+      err = hwloc_utils_enable_input_format(topology, flags, input, &input_format, verbose, callname); \
       if (err) return EXIT_FAILURE; \
     } \
     err = hwloc_topology_load(topology); \
@@ -288,7 +303,11 @@ int main(int argc, char *argv[])
 	return EXIT_SUCCESS;
       }
       if (!strcmp (argv[0], "--no-smt")) {
-	no_smt = 1;
+	no_smt = 0;
+	goto next;
+      }
+      if (!strncmp(argv[0], "--no-smt=", 9)) {
+	no_smt = atoi(argv[0] + 9);
 	goto next;
       }
       if (!strcmp (argv[0], "--restrict")) {
@@ -449,6 +468,11 @@ int main(int argc, char *argv[])
  next:
     argc--;
     argv++;
+
+    if (showobjs && nodeseto) {
+      fprintf(stderr, "ignoring --nodeset-output when --largest output is enabled\n");
+      nodeseto = 0;
+    }
   }
 
   ENSURE_LOADED();
@@ -478,6 +502,10 @@ int main(int argc, char *argv[])
 	*next = '\0';
       if (hwloc_calc_type_depth(topology, tmp, &hierdepth[i], "--hierarchical") < 0)
 	goto out;
+      if (hierdepth[i] < 0) {
+	fprintf(stderr, "unsupported (non-normal) --hierarchical type %s\n", tmp);
+	goto out;
+      }
       tmp = next+1;
     }
   }
