@@ -200,7 +200,7 @@ int main(int argc, char *argv[])
     cpuset = malloc(n * sizeof(hwloc_bitmap_t));
 
     if (input) {
-      err = hwloc_utils_enable_input_format(topology, input, &input_format, verbose, callname);
+      err = hwloc_utils_enable_input_format(topology, flags, input, &input_format, verbose, callname);
       if (err) {
 	free(cpuset);
 	return EXIT_FAILURE;
